# This script calculates timeseries of the maximum azimuthal wind speed, vmax, the radius where vmax occurs, RMW, 
#and the azimuthally-averaged tangential and radial winds from the 200-day axisymmetric simulations. The code
#also saves any data that could be used to construct Eulerian streamfunctions from any period during the 
#simulations. Note that this code is written for use on effectively 2-dimensional output data from axisymmetric 
#simulations. If the user desires to use this code on 3-dimensions, non-axisymmetric simulations, then changes 
#must be made


##Load modules
import numpy as np
from pylab import *
import netCDF4 as n4
import matplotlib.pyplot as plt
from matplotlib import gridspec
from math import *
import xarray as xr
from scipy import integrate
from scipy import spatial
from statistics import stdev
from shapely.geometry import LineString, Point
from PIL import Image
import os    

#Define functions

#Calculate Eulerian streamfunction
def streamfun(radwind,wwind,rhofield,rfield,drfield,zfield,dzfield):
#INPUTS:
#radwind, wwind = radial and vertical wind speeds (m/s)
#rhofield = air density (kg/m3)
#rfield, zfield = radial and vertical gridpoints (m)
#drfield, dzfield = radial and vertical grid spacings (m)
#All inputs must have dimensions (r, z)
###################################################################
#OUTPUTS:
#streamfield= Eulerian streamfunction as a function of r and z
###################################################################
    Psi_r = -np.cumsum(rfield*rhofield*wwind*drfield,axis=1)
    Psi_z = np.cumsum(rfield*rhofield*radwind*dzfield,axis=0)
    streamfield = 0.5*(Psi_r + Psi_z)
    return streamfield


SST=295 #Specify the SST used for the experiment from which we want to load the data
#Specify the location of the files- the code below accomodates the specific folder-naming system that I used for the simulations.
if SST-floor(SST)==0: #Case where the SST is an integer
    root="/scratch/groups/oneillm/Laurel/SST_exps_3D/Final_exps/RCE/"+np.str(SST)+"/"
elif SST-floor(SST)==0.5: #Case where the SST ends in ".5"
    root="/scratch/groups/oneillm/Laurel/SST_exps_3D/Final_exps/RCE/"+np.str(floor(SST))+"_5/"
#print(root)
#Load data from a random file to get all radial gridpoints from the simulation:
fileloc = root + 'cm1out_000100.nc'
ds1=xr.open_dataset(fileloc)
x_full=np.squeeze(ds1['xh']*1000).values
#Use the radial gridpoints in x_full to find the index corresponding to the outer radius specified by "Louter"
Louter=600 #Outer radius (km) for the MAFALDA analysis process
L_o_ind=min(np.where(x_full>Louter*1000)[0]) #Index corresponding to the outer radius
tstart=1 #First timestep from which we want to load the data. Set to 1 to begin at the start of the CM1 simulation. 
tfinal=2401 #Has value one greater than the timestep of the last file that we want to load data from. 
x0=0; x1=L_o_ind; z0 = 0; z1=20 #Minimum and maximum r and z values (km) for the data that we load. To load all 
                                #data, set z1 greater than or equal to the value of z at the model top and x1 
                                #greater than or equal to the full radial extent of the domain.
for ind in np.arange(tstart, tfinal): #Step through data files, startung at the timestep specified by "tstart" 
    if (ind < 1000)&(ind >= 100): 
        #print(ind)
        fileloc = root + 'cm1out_000'+np.str(ind)+'.nc'
    elif (ind < 100)&(ind >= 10): 
        #print(ind)
        fileloc = root + 'cm1out_0000'+np.str(ind)+'.nc'     
    elif ind<10:
        #print(ind)
        fileloc = root + 'cm1out_00000'+np.str(ind)+'.nc'        
    else:
        #print(ind)
        fileloc = root + 'cm1out_00'+np.str(ind)+'.nc'
    ds=xr.open_dataset(fileloc) #Open the netcdf file corresponding to the current timestamp
    if ind==tstart: #Initialize all data arrays for the special case of ind=tstart
        #Initialize output variables
        temp_v=np.squeeze(ds['v10'].sel(xh=slice(x0,x1))).values #Azimuthal wind speed at z=10m
        temp_v=temp_v[..., np.newaxis]
        v=temp_v #Dims: z * y * x 
        
        temp_u=np.squeeze(ds['u10'].sel(xh=slice(x0,x1))).values #Radial wind speed at z=10m
        temp_u=temp_u[..., np.newaxis]
        u=temp_u

        temp_w=np.squeeze(ds['winterp'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values #Vertical wind speed
        temp_w=temp_w[..., np.newaxis]
        w=temp_w

        temp_rho=np.squeeze(ds['rho'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values #Dry-air density
        temp_rho=temp_rho[..., np.newaxis]
        rho=temp_rho

        temp_u2=np.squeeze(ds['uinterp'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values #Radial wind speed
        temp_u2=temp_u2[..., np.newaxis]
        u2=temp_u2

        temp_v2=np.squeeze(ds['vinterp'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values #Azimuthal wind speed
        temp_v2=temp_v2[..., np.newaxis]
        v2=temp_v2

        z=np.squeeze(ds['zh'].sel(zh=slice(z0,z1))*1000).values #Vertical gridpoints. Convert to m
        
        x=np.squeeze(ds['xh'].sel(xh=slice(x0,x1))*1000).values #Radial gridpoints. Convert to m

        dx=np.mean(np.diff(x)) #Radial grid spacing, assumed constant
        
        time=np.squeeze(ds['time']) #Current time in days, hours, minutes, and seconds
    
        #For the cases where the output variables have already been initialized, append data (with dimensions (z, x, y) or (x, y)) 
        #from the current timestamp to the existing arrays so they have dimensions (z, x, y, t) or (x, y, t)    
    else:
   
        #Append new data to output variables 
        temp_v=np.squeeze(ds['v10'].sel(xh=slice(x0,x1))).values
        temp_v=temp_v[..., np.newaxis]
        v=np.concatenate((v, temp_v), axis=1)
        
        temp_u=np.squeeze(ds['u10'].sel(xh=slice(x0,x1))).values
        temp_u=temp_u[..., np.newaxis]
        u=np.concatenate((u, temp_u), axis=1)       

        temp_v2=np.squeeze(ds['vinterp'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values
        temp_v2=temp_v2[..., np.newaxis]
        v2=np.concatenate((v2, temp_v2), axis=2)

        temp_u2=np.squeeze(ds['uinterp'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values
        temp_u2=temp_u2[..., np.newaxis]
        u2=np.concatenate((u2, temp_u2), axis=2)

        temp_w=np.squeeze(ds['winterp'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values
        temp_w=temp_w[..., np.newaxis]
        w=np.concatenate((w, temp_w), axis=2)

        temp_rho=np.squeeze(ds['rho'].sel(xh=slice(x0,x1), zh=slice(z0, z1))).values
        temp_rho=temp_rho[..., np.newaxis]
        rho=np.concatenate((rho, temp_rho), axis=2)
    

#Get timeseries of maximum azimuthal winds (vmax, m/s) and radius of maximum azimuthal winds (RMW, km)
vmax=[]
RMW=[]
for i in np.arange(tfinal-tstart):
    vmax_t=np.max(v[:, i])
    vmax.append(vmax_t)
    RMW_ind=np.where(v[:, i]==vmax_t)[0][0]
    RMW.append(x[RMW_ind]/1000) #Convert to km
#OPTIONAL- Plot vmax and RMW versus time 
plt.figure(figsize=(15,10)) 
time_plot=(np.arange(tfinal-tstart)+tstart)/12 #Time in days
plt.plot(time_plot, vmax)
plt.ylim(0,100)
plt.xlabel("Time (days)")
plt.ylabel("vmax (m/s)")
plt.title("vmax (m/s) versus time")
plt.savefig("vmax_0.png")
plt.figure(figsize=(15,10)) 
plt.plot(time_plot, np.asarray(RMW)) 
plt.xlabel("Time (days)")
plt.ylabel("RMW (km)")
plt.title("RMW (km) versus time")
plt.ylim(0,200)
plt.savefig("RMW_0.png")
#Save vmax, RMW, and data that can be used to make Hovmoller plots for the simulation
np.savez("vmax_RMW_"+np.str(SST)+".npz", vmax=vmax, RMW=RMW) 
np.savez("Hovmoller_"+np.str(SST)+".npz", v=v, u=u, x=x, time_plot=time_plot)

#Pad the vectors containing the radial and vertical gridpoints and grid spacing so that they 
#all have shape (r, z)
r=np.expand_dims(x, axis=1)
dr1=np.diff(x)
dr_l=dr1.tolist()
dr_l.append(dr_l[-1])
dr=np.asarray(dr_l)
drn=np.expand_dims(dr, axis=1)
for i in np.arange(len(z)):
    if i==0:
        r_padded=r
        dr_padded=drn
    else:
        r_padded=np.concatenate((r_padded, r), axis=1)
        dr_padded=np.concatenate((dr_padded, drn), axis=1)
r_pad=np.transpose(r_padded)
dr_pad=np.transpose(dr_padded)
zn=np.expand_dims(z, axis=1)
dz1=np.diff(z)
dz_l=dz1.tolist()
dz_l.append(dz_l[-1])
dz=np.asarray(dz_l)
dzn=np.expand_dims(dz, axis=1)
for i in np.arange(len(x)):
    if i==0:
        z_padded=zn
        dz_padded=dzn
    else:
        z_padded=np.concatenate((z_padded, zn), axis=1)
        dz_padded=np.concatenate((dz_padded, dzn), axis=1)
#Save data that can be used to calculate the Eulerian streamfunction
np.savez("Streamf_inputs_"+np.str(SST)+".npz", x=x, z=z, uwind=u2, wwind=w, rho=rho, r_pad=r_pad, dr_pad=dr_pad, z_pad=z_padded, dz_pad=dz_padded)
